class Candidate {
  
  // define variables
  float x; // x position
  float y; // y position
  float tempW; // to help figure out width
  float w; // width
  float h; // height
  
  String name; // candidate name
  int riding; // candidate riding
  float votes; // number of votes
  int rank; // candidate rank within riding
  color colorValue; // fill color
  float money; // how much money contributed
  float rangeMoney; // altered money value for filtered results
  
  StringList contributors; // names of all the contributors
  float[] contributedAmounts; // array for all the contributor's donation amounts
  
  float contributorX; // start x position for contributor bar
  float contributorY; // start y position for contributor bar
  float contributorW; // width of contributor bar
  float contributorH; // height of contributor bar
  
  boolean thisHovering; // if hovering on this variable
  
  
  Candidate(String tempName, int tempRiding, float tempVotes) {
    // assign variable values
    name = tempName;
    riding = tempRiding;
    votes = tempVotes;
    money = 0;
    
    contributors = new StringList();
    contributorW = 300;
    contributorX = textX - (contributorW/2);
    contributorY = 520;
    contributorH = 15;
    
  } // end class declaration

  void fixWidth() {
    if (money > 0) {
        tempW = money;
      } else {
        tempW = 1000; // value so candidates with $0 can still be seen
      }
   
      // fix for money values that'll distort the range filter
      if (money > moneyMax) {
        rangeMoney = moneyMax;
      } else {
        rangeMoney = money;
      }
  
  } // end fixWidth function

  void contributorAmounts() {
    // figure out how much each contributor has donated to this candidate
    
    // declare size of float[]
    contributedAmounts = new float[contributors.size()];
    
    // search appropriate table data
    if (riding == 0) {
      for (int row = 0; row < cMRowCount; row++) {
        // assign temp variables
        String tempName = cMData.getString(row, "contributorName");
        String tempCandidate = cMData.getString(row, "candidate");
        float tempAmount = cMData.getFloat(row, "amount");
        // if candidate names match, run through contributors StringList for match
        // then add contributed amount to matching float[] index
        if (tempCandidate.equals(name)) {
          for (int n = 0; n < contributors.size(); n++) {
            String thisName = contributors.get(n);
            if (thisName.equals(tempName)) {
              contributedAmounts[n] += tempAmount;
            }
          }
        }
      }
    } else {
      for (int row = 0; row < cWRowCount; row++) {
        // assign temp variables
        String tempName = cWData.getString(row, "contributorName");
        tempName = trim(tempName);
        String tempCandidate = cWData.getString(row, "candidate");
        float tempAmount = cWData.getFloat(row, "amount");
        // if candidate names match, run through contributors StringList for match
        // then add contributed amount to matching float[] index
        if (tempCandidate.equals(name)) {
          for (int n = 0; n < contributors.size(); n++) {
            String thisName = contributors.get(n);
            if (thisName.equals(tempName)) {
              contributedAmounts[n] += tempAmount;
            }
          }
        }
      }
    }
  
  } // end contributorAmounts function

  
  void order() {
    // function to call once the rank has been assigned
    // and candidates can be ordered within their ridings
    
    // determine color based on even/odd modulo
    if (rank % 2 == 0) {
      // even number
      colorValue = scheme[0];
    } else {
      // odd number
      colorValue = scheme[1];
    }
    
    // determine width based off largest value (separate scales for Mayoral and Ward ridings)
    if (riding == 0) {
      w = tempW  * iWidth / mScale;
    } else {
      w = tempW  * iWidth / wScale;
    }
    // determine height based of number of ridings
    h = iHeight/ridingCandidates.length;
    // determine y position based off riding and height, and with scaleSeparator
    if (riding == 0) {
      y = ((h + view1Spacing) * riding) + view1StartY;
    } else {
      y = ((h + view1Spacing) * riding) + view1StartY + scaleSeparator;
    }
    
    
  } // end order function

  void display1() {
    // First display view: Normal bar graph
    
    // reset contributor bar X variable
    contributorX = textX - (contributorW/2);
    
    // if on hover, gray out & display texts
    if (mouseX > x && mouseX < x+w && mouseY > y && mouseY < y+h) {
      thisHovering = true; // show sidebar text
      if (riding == 0) {
        sWard = "in Mayoral race";
      } else {
        sWard = "in Ward " + riding;
      }
      sRank = rank;
      sName = name;
      sContributors = contributors.size();
      sMoney = money;
      // gray out rectangle
      fill(darkGrey);
    } else {
      thisHovering = false; // hide sidebar text
      // change color value if candidate is in filter range
      if (rangeMoney >= rangeMin && rangeMoney <= rangeMax) {
        fill(colorValue);
      } else {
        fill(lightGrey);
      }
    }
    
    noStroke();
    rect(x, y, w, h);
    
  } // end display1 function


  void display2(int tempPosition) {
    // second display view
    
    //x = view2StartX + (riding * view2Spacing);
    x = sidebarW + view2Padding + ((view2Spacing + view2Padding/2) * tempPosition);
    y = ((h + 3) * rank) + view2StartY; // 5px away from top
    
    if (riding == 0) {
      w = tempW  * view2Spacing / mScale;
    } else {
      w = tempW  * view2Spacing / wScale;
    }
    
    // reset contributor bar X variable
    contributorX = textX - (contributorW/2);
    
    // if on hover, gray out & display texts
    // different hover for display2, because of values that are too small to register
    if (mouseX > x && mouseX < x+(view2Spacing-view2Padding) && mouseY > y && mouseY < y+h) {
      thisHovering = true;
      if (riding == 0) {
        sWard = "in Mayoral race";
      } else {
        sWard = "in Ward " + riding;
      }
      sRank = rank;
      sName = name;
      sContributors = contributors.size();
      sMoney = money;
      // background rectangle
      fill(lightGrey);
      rect(x, y, view2Spacing-view2Padding, h);
      // gray out rectangle
      fill(darkGrey);
    } else {
      thisHovering = false; // hide sidebar text
      // change color value if candidate is in filter range
      if (rangeMoney >= rangeMin && rangeMoney <= rangeMax) {
        fill(colorValue);
      } else {
        fill(lightGrey);
      }
    }
    
    noStroke();
    rect(x, y, w, h);
    
    // legend
    textAlign(RIGHT);
    fill(darkGrey);
    noStroke();
    textSize(8);
    text(rank+1, x - 5, y+(h-1));
    

  }  // end display2 function


  void contributorBar() {
    // Function to show contributor bar on sidebar
    fill(black);
    rect(contributorX, contributorY, contributorW, contributorH); // black placeholder
    for(int i = 0; i < contributedAmounts.length; i++) {
      if (i % 2 == 0) {
        fill(scheme[0]);
      } else {
        fill(scheme[1]);
      }
      float thisWidth = contributedAmounts[i] * contributorW / money;
      rect(contributorX, contributorY, thisWidth, contributorH);
      contributorX += thisWidth;
    }
  } // end contributorBar function




} // end class
