import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class barkunJ_project2 extends PApplet {

// Icicle tree

// range slider

ControlP5 cp5;
Range range;
DropdownList d1, d2;
boolean toggleValue = true;
float moneyMin = MAX_INT;
float moneyMax = 100000;
float rangeMin;
float rangeMax;

// data tables
Table cMData; // Contribution Mayor data
Table pMData; // Poll Mayor data
Table cWData; // Contribution Ward data
Table pWData; // Poll Ward data
int cMRowCount;
int pMRowCount;
int cWRowCount;
int pWRowCount;

// variables from data
String cName; // candidate name
int cWard; // candidate ward
float cVotes; // candidate votes

// calculated data variables
int[] ridingCandidates = new int[45]; // number of candidates per each riding (44 + mayor)
float[] totalMoney = new float[45]; // total money contibuted per riding
float[] totalWidth = new float[45]; // total width of each riding to find the longest one and adjust scale
float mScale = MIN_INT; // calculating the scale for mayoral riding
float wScale = MIN_INT; // calculating the scale for ward riding
float largestWidth = MIN_INT; // largest width for view2 spacing

// declare Candidate class
Candidate[] candidates;

// assorted variables
float iWidth; // width of data visualization block
float iHeight; // height of data visualization block - mayoral campaign
float scaleSeparator = 50; // space between mayoral and ward campaigns

// colors
int[] scheme1 = {0xff79BD8F, 0xff00A388}; // first color scheme
int[] scheme = scheme1; // chosen color scheme
int black = 0xff151515;
int darkGrey = 0xff282828;
int grey = 0xffCBCBCB;
int lightGrey = 0xffEDEDED;
int medGrey = 0xff838383;
int white = 0xffffffff;

// sidebar variables
float sidebarW = 330; // sidebar width
boolean isHovering; // if true, show sidebar details
float textX = sidebarW/2; // X position for centered text
int sRank = 0; // sidebar rank
String sName = ""; // sidebar name
String sWard = ""; // sidebar ward #
int sContributors = 0; // sidebar # of contributors
float sMoney = 0; // sidebar money

// view-specific variables
float view1Spacing = 4;
float view1StartY = 60;
float view1StartX = sidebarW + 60;
float view2Spacing;
float view2Padding = 60;
float view2StartX = sidebarW + view2Padding - 20; // view 2 x starting point
float view2StartY = 100; // view 2 space from top

// font variables
PFont font28;
PFont font24;
PFont font16;
PFont font14;
PFont font9;
ControlFont fontList;

public void setup() {
  size(1200, 800);
  background(255);
  smooth();

  iWidth = width - 420;
  iHeight = height - 310;
  isHovering = false;
  view2Spacing = (width - sidebarW) / 3 - (view2Padding/2);
  
  font28 = loadFont("FrancoisOne-Regular-28.vlw");
  font24 = loadFont("FrancoisOne-Regular-24.vlw");
  font16 = loadFont("Lato-Regular-16.vlw");
  font14 = loadFont("Lato-Regular-14.vlw");
  font9 = loadFont("Lato-Bold-9.vlw");
  fontList = new ControlFont(font16);

  // load mayoral data
  cMData = loadTable("contribution-1.tsv", "header");
  pMData = loadTable("poll-1.tsv", "header");
  cMRowCount = cMData.getRowCount();
  pMRowCount = pMData.getRowCount();

  // load ward data
  cWData = loadTable("contribution-2.tsv", "header");
  pWData = loadTable("poll-2.tsv", "header");
  cWRowCount = cWData.getRowCount();
  pWRowCount = pWData.getRowCount();

  // declare candidate array size
  candidates = new Candidate[pMRowCount + pWRowCount];

  // setting up candidate class (Mayor)  
  for (int row = 0; row < pMRowCount; row++) {
    cVotes = 0; // reset vote counter for new candidate
    cName = pMData.getString(row, "candidate");
    cName = trim(cName); // trim any extra spaces
    cWard = 0;
    for (int riding = 1; riding < 44; riding++) {
      // add all ward votes to voting total
      float tempVote = pMData.getFloat(row, "ward" + riding);
      cVotes += tempVote;
    }
    // add to class array
    candidates[row] = new Candidate(cName, cWard, cVotes);
    // add +1 to total number of candidates per that riding
    ridingCandidates[cWard] += 1;
  }

  // setting up candidate class (Wards)
  for (int row = 0; row < pWRowCount; row++) {
    cVotes = 0; // reset vote counter for new candidate
    cName = pWData.getString(row, "candidate");
    cName = trim(cName); // trim any extra spaces
    cWard = pWData.getInt(row, "ward");
    cVotes = pWData.getFloat(row, "votes");
    // add to class array
    candidates[pMRowCount + row] = new Candidate(cName, cWard, cVotes);
    // add +1 to total number of candidates per that riding
    ridingCandidates[cWard] += 1;
  }

  // set up candidate rank within riding
  // for each riding
  for (int r = 0; r < 45; r++) {
    int i = 0;
    // set up new array with # of candidates for that riding
    float[] allVotes = new float[ridingCandidates[r]];
    // run through candidate objects for matching riding
    for (int c = 0; c < candidates.length; c++) {
      // if ridings match
      if ( candidates[c].riding == r) {
        // add vote # to array
        allVotes[i] = candidates[c].votes;
        i++;
      }
    }
    // sort resulting array
    allVotes = sort(allVotes);
    allVotes = reverse(allVotes);
    // run through array
    for (int n = 0; n < allVotes.length; n++) {
      // run through candidates for match
      for (int c = 0; c < candidates.length; c++) {
        if (candidates[c].riding == r && candidates[c].votes == allVotes[n]) {
          // if it's the same candidate as originally, store new rank value
          candidates[c].rank = n;
          break;
        }
      }
    }
  }

  // add money values and contributor names to candidate classes
  // mayor
  for (int row = 0; row < cMRowCount; row++) {
    boolean nameMatch = false;
    String tempName = cMData.getString(row, "candidate");
    tempName = trim(tempName);
    String tempCName = cMData.getString(row, "contributorName");
    tempCName = trim(tempCName);
    float tempAmount = cMData.getFloat(row, "amount");
    int tempWard = cMData.getInt(row, "ward");
    // run through all candidates
    // if names and wards match, add tempAmount to money value & contributor name to StringList
    for (int c = 0; c < candidates.length; c++) {
      if (tempName.equals(candidates[c].name) && tempWard == candidates[c].riding) {
        candidates[c].money += tempAmount;
        nameMatch = true;
        if (!candidates[c].contributors.hasValue(tempCName)) {
          candidates[c].contributors.append(tempCName);
        }
      }
    }
  }
  // wards
  for (int row = 0; row < cWRowCount; row++) {
    boolean nameMatch = false;
    String tempName = cWData.getString(row, "candidate");
    tempName = trim(tempName);
    String tempCName = cWData.getString(row, "contributorName");
    float tempAmount = cWData.getFloat(row, "amount");
    int tempWard = cWData.getInt(row, "ward");
    // run through all candidates
    // if names and wards match, add tempAmount to money value & contributor name to StringList
    for (int c = 0; c < candidates.length; c++) {
      if (tempName.equals(candidates[c].name) && tempWard == candidates[c].riding) {
        candidates[c].money += tempAmount;
        nameMatch = true;
        if (!candidates[c].contributors.hasValue(tempCName)) {
          candidates[c].contributors.append(tempCName);
        }
      }
    }
  }

  // go through candidates
  for (int c = 0; c < candidates.length; c++) {
    // fix the width and money
    candidates[c].fixWidth();
    // add money to total amount per ward
    // figure out moneyMin and moneyMax for range slider
    totalMoney[candidates[c].riding] += candidates[c].money;
    if (candidates[c].money < moneyMin) {
      moneyMin = candidates[c].money;
    }
    // total width of each riding bar
    totalWidth[candidates[c].riding] += candidates[c].tempW;
  }
  
  // determine scale of both riding categories
  mScale = totalWidth[0];
  for (int i = 1; i < totalWidth.length; i++) {
    if (totalWidth[i] > wScale) {
      wScale = totalWidth[i];
    }
  }
  println(mScale);
  println(wScale);


  // make range Min & Max equal to money amounts to start
  rangeMin = moneyMin;
  rangeMax = moneyMax;

  // once ranks and money have been assigned:
  // order candidates
  // figure out contributor details
  // find largest width
  for (int c = 0; c < candidates.length; c++) {
    candidates[c].order(); 
    candidates[c].contributorAmounts();
    if (candidates[c].w > largestWidth) {
      largestWidth = candidates[c].w;
    }
  }

  // determine x pos of each candidate
  for (int c = 0; c < candidates.length; c++) {
    float tempX = view1StartX;
    int thisRank = candidates[c].rank;
    int thisRiding = candidates[c].riding;
    for (int i = 0; i < thisRank; i++) {
      // find each candidate before this one in rank
      for (int n = 0; n < candidates.length; n++) {
        if (candidates[n].riding == thisRiding && candidates[n].rank == i) {
          tempX += candidates[n].w;
        }
      }
    }
    // set candidate x position
    candidates[c].x = tempX;
  }

  // setup range selector
  cp5 = new ControlP5(this);
  range = cp5.addRange("rangeController")
    // disable broadcasting since setRange and setRangeValues will trigger an event
    .setBroadcast(false) 
    .setPosition(textX-150, 220)
    .setSize(300, 20)
    .setHandleSize(20)
    .setLabelVisible(true)
    .setCaptionLabel("")
    .setRange(moneyMin, moneyMax)
    .setRangeValues(moneyMin, moneyMax)
    .setColorCaptionLabel(white)
    // after the initialization we turn broadcast back on again
    .setBroadcast(true)
    .setColorActive(scheme[1])
    .setColorForeground(scheme[0])
    .setColorBackground(black)  
    ;

  // setup toggle
  cp5.addToggle("toggle")
    .setPosition(textX-(sidebarW/2), 0)
    .setSize(PApplet.parseInt(sidebarW), 20)
    .setValue(true)
    .setMode(ControlP5.SWITCH)
    .setColorBackground(black)
    .setColorActive(medGrey) 
    .setLabelVisible(false)
    ;
    
    // setup dropdowns
    d1 = cp5.addDropdownList("compare1")
      .setPosition(sidebarW + view2Padding + (view2Spacing + view2Padding/2), view2StartY - 25)
      ;
      
    d2 = cp5.addDropdownList("compare2")
      .setPosition(sidebarW + view2Padding + ((view2Spacing + view2Padding/2) * 2), view2StartY - 25)
      ;
      
    customize(d1);
    customize(d2);
    d1.setIndex(0);
    d2.setIndex(1);
    
    
} // end setup


public void draw() {
  background(255);
  isHovering = false; // reset boolean

  // display candidates
  for (int c = 0; c < candidates.length; c++) {
    if (toggleValue == true) {
      candidates[c].display1();
      d1.setVisible(false);
      d2.setVisible(false);
    } else {
      // show mayoral candidates
      if (candidates[c].riding == 0) {
        candidates[c].display2(0);
      } else if (candidates[c].riding == d1.getValue()) {
        candidates[c].display2(1);
      } else if (candidates[c].riding == d2.getValue()) {
        candidates[c].display2(2);
      }
      d1.setVisible(true);
      d2.setVisible(true);
    }
    // check for hovering
    if (candidates[c].thisHovering) {
      isHovering = true;
    }
    
  } // end for block

  // sidebar
  fill(darkGrey);
  rect(0, 0, sidebarW, height);

  // dividing lines
  stroke(black);
  strokeWeight(2);
  line(textX-40, 300, textX+40, 300);

  // text placeholders for candidate-specific data
  if (isHovering) {
    fill(white);
    noStroke();
    textAlign(CENTER);
    // rank
    textFont(font24, 24);
    text("Ranked #" + (sRank + 1), textX, 380);
    // ward # and candidate name
    textFont(font14, 14);
    text(sWard, textX, 400);
    textFont(font16, 16);
    text(sName, textX, 440);
    // contributors and amount
    text(nfc(sContributors) + " contributors", textX, 480);
    text("$" + nfc(sMoney, 2) + " in contributions", textX, 500);
    textFont(font9, 9);
    text("Percentage of total contributions per contributor", textX, 550);
    // contributor bar
    for (int c = 0; c < candidates.length; c++) {
      if (candidates[c].thisHovering) {
        candidates[c].contributorBar();
      }
    }
  } else {
    noStroke();
    fill(white);
    textAlign(CENTER);
    textFont(font24, 24);
    text("Hover", textX, 380);
    textFont(font14, 14);
    text("on visualization for details", textX, 400);
  }

  // view-specific legends
  if (toggleValue == true) {
    // view 1 legends
    textAlign(LEFT);
    fill(darkGrey);
    noStroke();
    textFont(font16, 16);
    text("$ Contributed to Mayoral Candidates", view1StartX, view1StartY - 30);
    text("$ Contributed to Ward Candidates", view1StartX, view1StartY + candidates[0].h + scaleSeparator - 20);
    textAlign(RIGHT);
    for (int i = 0; i < 45; i++) {
      textFont(font9, 9);
      if (i == 0) { 
      } else {
        text(i, view1StartX - 5, view1StartY + candidates[0].h - 1 + (i * (candidates[0].h + view1Spacing)) + scaleSeparator);
      }
    }
    // scale legends
    fill(darkGrey);
    textAlign(LEFT);
    text("$0", view1StartX, view1StartY - 8);
    text("$0", view1StartX, view1StartY + candidates[0].h + scaleSeparator - 3);
    textAlign(RIGHT);
    text("$" + nfc(mScale, 2), view1StartX + iWidth, view1StartY - 8);
    text("$" + nfc(wScale, 2), view1StartX + iWidth, view1StartY + candidates[0].h + scaleSeparator - 3);
    stroke(lightGrey);
    strokeWeight(1);
    line(view1StartX, view1StartY - 5, view1StartX + iWidth, view1StartY - 5);
    line(view1StartX, view1StartY + candidates[0].h + scaleSeparator, view1StartX + iWidth, view1StartY + candidates[0].h + scaleSeparator);
    textAlign(CENTER);
    fill(white);
    noStroke();
    textFont(font14, 14);
    text("Overview", textX, 45); 
  } else {
    // view 2 legends
    textAlign(LEFT);
    fill(darkGrey);
    noStroke();
    textFont(font16, 16);
    text("Mayoral", sidebarW + view2Padding, view2StartY - 25);
    stroke(grey);
    strokeWeight(1);
    line(sidebarW + view2Spacing + (view2Padding/2), 50, sidebarW + view2Spacing + (view2Padding/2), height-50);
    // scale legends
    stroke(lightGrey);
    line(sidebarW + view2Padding, view2StartY - 5, sidebarW + view2Spacing, view2StartY - 5);
    line(sidebarW + view2Spacing + (view2Padding * 1.5f), view2StartY - 5, sidebarW + (view2Spacing*2) + (view2Padding/2), view2StartY - 5);
    line(sidebarW + view2Spacing*2 + (view2Padding * 2), view2StartY - 5, sidebarW + (view2Spacing*3) + view2Padding, view2StartY - 5);
    noStroke();
    fill(darkGrey);
    textFont(font9, 9);
    text("$0", sidebarW + view2Padding, view2StartY - 8);
    text("$0", sidebarW + view2Spacing + (view2Padding * 1.5f), view2StartY - 8);
    text("$0", sidebarW + view2Spacing*2 + (view2Padding * 2), view2StartY - 8);
    textAlign(RIGHT);
    text("$ " + nfc(mScale, 2), sidebarW + view2Spacing, view2StartY - 8);
    text("$ " + nfc(wScale, 2), sidebarW + (view2Spacing*2) + (view2Padding/2), view2StartY - 8);
    text("$ " + nfc(wScale, 2), sidebarW + (view2Spacing*3) + view2Padding, view2StartY - 8);
    textAlign(CENTER);
    fill(white);
    noStroke();
    textFont(font14, 14);
    text("Details", textX, 45); 
  }

  // static text
  textAlign(CENTER);
  fill(white);
  noStroke();
  textFont(font28, 28);
  text("2010 Toronto Elections", textX, 120);
  textFont(font14, 14);
  text("Poll ranks & campaign contributions", textX, 145);
  text("Drag slider to filter contributed amount", textX, 210);
  
} // end draw



public void controlEvent(ControlEvent theControlEvent) {
  if (theControlEvent.isFrom("rangeController")) {
    // min and max values are stored in an array.
    // access this array with controller().arrayValue().
    // min is at index 0, max is at index 1.
    rangeMin = PApplet.parseInt(theControlEvent.getController().getArrayValue(0));
    rangeMax = PApplet.parseInt(theControlEvent.getController().getArrayValue(1));
  }
} // end controlEvent


public void toggle(boolean theFlag) {
  if (theFlag==true) {
    // view 1
    toggleValue = true;
    // fix values
    for (int c = 0; c < candidates.length; c++) {
      candidates[c].order();
    }
    // determine x pos of each candidate
    for (int c = 0; c < candidates.length; c++) {
      float tempX = view1StartX;
      int thisRank = candidates[c].rank;
      int thisRiding = candidates[c].riding;
      for (int i = 0; i < thisRank; i++) {
        // find each candidate before this one in rank
        for (int n = 0; n < candidates.length; n++) {
          if (candidates[n].riding == thisRiding && candidates[n].rank == i) {
            tempX += candidates[n].w;
          }
        }
      }
      // set candidate x position
      candidates[c].x = tempX;
    } // end for block
  } else {
    // view 2
    toggleValue = false;
  }
} // end toggle


public void customize(DropdownList ddl) {
  // a convenience function to customize a DropdownList
  ddl.setBackgroundColor(grey);
  ddl.setItemHeight(20);
  ddl.setBarHeight(25);
  ddl.setHeight(200);
  ddl.captionLabel().set("Choose a ward");
  ddl.captionLabel().style().marginTop = 5;
  ddl.captionLabel().style().marginLeft = 3;
  ddl.valueLabel().style().marginTop = 3;
  for (int i=1;i<45;i++) {
    ddl.addItem("Ward "+i, i);
  }
  //ddl.scroll(0);
  ddl.setColorBackground(lightGrey);
  ddl.setColorActive(scheme[1]);
  ddl.setColorForeground(scheme[1]);
  ddl.setColorLabel(darkGrey);
  ddl.captionLabel()
     .setFont(fontList)
     .setSize(16)
     .toUpperCase(false)
     ;
  ddl.valueLabel()
    .setFont(fontList)
    .setSize(16)
    .toUpperCase(false)
    ;
}
class Candidate {
  
  // define variables
  float x; // x position
  float y; // y position
  float tempW; // to help figure out width
  float w; // width
  float h; // height
  
  String name; // candidate name
  int riding; // candidate riding
  float votes; // number of votes
  int rank; // candidate rank within riding
  int colorValue; // fill color
  float money; // how much money contributed
  float rangeMoney; // altered money value for filtered results
  
  StringList contributors; // names of all the contributors
  float[] contributedAmounts; // array for all the contributor's donation amounts
  
  float contributorX; // start x position for contributor bar
  float contributorY; // start y position for contributor bar
  float contributorW; // width of contributor bar
  float contributorH; // height of contributor bar
  
  boolean thisHovering; // if hovering on this variable
  
  
  Candidate(String tempName, int tempRiding, float tempVotes) {
    // assign variable values
    name = tempName;
    riding = tempRiding;
    votes = tempVotes;
    money = 0;
    
    contributors = new StringList();
    contributorW = 300;
    contributorX = textX - (contributorW/2);
    contributorY = 520;
    contributorH = 15;
    
  } // end class declaration

  public void fixWidth() {
    if (money > 0) {
        tempW = money;
      } else {
        tempW = 1000; // value so candidates with $0 can still be seen
      }
   
      // fix for money values that'll distort the range filter
      if (money > moneyMax) {
        rangeMoney = moneyMax;
      } else {
        rangeMoney = money;
      }
  
  } // end fixWidth function

  public void contributorAmounts() {
    // figure out how much each contributor has donated to this candidate
    
    // declare size of float[]
    contributedAmounts = new float[contributors.size()];
    
    // search appropriate table data
    if (riding == 0) {
      for (int row = 0; row < cMRowCount; row++) {
        // assign temp variables
        String tempName = cMData.getString(row, "contributorName");
        String tempCandidate = cMData.getString(row, "candidate");
        float tempAmount = cMData.getFloat(row, "amount");
        // if candidate names match, run through contributors StringList for match
        // then add contributed amount to matching float[] index
        if (tempCandidate.equals(name)) {
          for (int n = 0; n < contributors.size(); n++) {
            String thisName = contributors.get(n);
            if (thisName.equals(tempName)) {
              contributedAmounts[n] += tempAmount;
            }
          }
        }
      }
    } else {
      for (int row = 0; row < cWRowCount; row++) {
        // assign temp variables
        String tempName = cWData.getString(row, "contributorName");
        tempName = trim(tempName);
        String tempCandidate = cWData.getString(row, "candidate");
        float tempAmount = cWData.getFloat(row, "amount");
        // if candidate names match, run through contributors StringList for match
        // then add contributed amount to matching float[] index
        if (tempCandidate.equals(name)) {
          for (int n = 0; n < contributors.size(); n++) {
            String thisName = contributors.get(n);
            if (thisName.equals(tempName)) {
              contributedAmounts[n] += tempAmount;
            }
          }
        }
      }
    }
  
  } // end contributorAmounts function

  
  public void order() {
    // function to call once the rank has been assigned
    // and candidates can be ordered within their ridings
    
    // determine color based on even/odd modulo
    if (rank % 2 == 0) {
      // even number
      colorValue = scheme[0];
    } else {
      // odd number
      colorValue = scheme[1];
    }
    
    // determine width based off largest value (separate scales for Mayoral and Ward ridings)
    if (riding == 0) {
      w = tempW  * iWidth / mScale;
    } else {
      w = tempW  * iWidth / wScale;
    }
    // determine height based of number of ridings
    h = iHeight/ridingCandidates.length;
    // determine y position based off riding and height, and with scaleSeparator
    if (riding == 0) {
      y = ((h + view1Spacing) * riding) + view1StartY;
    } else {
      y = ((h + view1Spacing) * riding) + view1StartY + scaleSeparator;
    }
    
    
  } // end order function

  public void display1() {
    // First display view: Normal bar graph
    
    // reset contributor bar X variable
    contributorX = textX - (contributorW/2);
    
    // if on hover, gray out & display texts
    if (mouseX > x && mouseX < x+w && mouseY > y && mouseY < y+h) {
      thisHovering = true; // show sidebar text
      if (riding == 0) {
        sWard = "in Mayoral race";
      } else {
        sWard = "in Ward " + riding;
      }
      sRank = rank;
      sName = name;
      sContributors = contributors.size();
      sMoney = money;
      // gray out rectangle
      fill(darkGrey);
    } else {
      thisHovering = false; // hide sidebar text
      // change color value if candidate is in filter range
      if (rangeMoney >= rangeMin && rangeMoney <= rangeMax) {
        fill(colorValue);
      } else {
        fill(lightGrey);
      }
    }
    
    noStroke();
    rect(x, y, w, h);
    
  } // end display1 function


  public void display2(int tempPosition) {
    // second display view
    
    //x = view2StartX + (riding * view2Spacing);
    x = sidebarW + view2Padding + ((view2Spacing + view2Padding/2) * tempPosition);
    y = ((h + 3) * rank) + view2StartY; // 5px away from top
    
    if (riding == 0) {
      w = tempW  * view2Spacing / mScale;
    } else {
      w = tempW  * view2Spacing / wScale;
    }
    
    // reset contributor bar X variable
    contributorX = textX - (contributorW/2);
    
    // if on hover, gray out & display texts
    // different hover for display2, because of values that are too small to register
    if (mouseX > x && mouseX < x+(view2Spacing-view2Padding) && mouseY > y && mouseY < y+h) {
      thisHovering = true;
      if (riding == 0) {
        sWard = "in Mayoral race";
      } else {
        sWard = "in Ward " + riding;
      }
      sRank = rank;
      sName = name;
      sContributors = contributors.size();
      sMoney = money;
      // background rectangle
      fill(lightGrey);
      rect(x, y, view2Spacing-view2Padding, h);
      // gray out rectangle
      fill(darkGrey);
    } else {
      thisHovering = false; // hide sidebar text
      // change color value if candidate is in filter range
      if (rangeMoney >= rangeMin && rangeMoney <= rangeMax) {
        fill(colorValue);
      } else {
        fill(lightGrey);
      }
    }
    
    noStroke();
    rect(x, y, w, h);
    
    // legend
    textAlign(RIGHT);
    fill(darkGrey);
    noStroke();
    textSize(8);
    text(rank+1, x - 5, y+(h-1));
    

  }  // end display2 function


  public void contributorBar() {
    // Function to show contributor bar on sidebar
    fill(black);
    rect(contributorX, contributorY, contributorW, contributorH); // black placeholder
    for(int i = 0; i < contributedAmounts.length; i++) {
      if (i % 2 == 0) {
        fill(scheme[0]);
      } else {
        fill(scheme[1]);
      }
      float thisWidth = contributedAmounts[i] * contributorW / money;
      rect(contributorX, contributorY, thisWidth, contributorH);
      contributorX += thisWidth;
    }
  } // end contributorBar function




} // end class
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "--full-screen", "--bgcolor=#666666", "--hide-stop", "barkunJ_project2" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
